package com.gic.plug.web.exception;

import com.gic.commons.exception.BaseRuntimeException;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.exception.CommonException;
import com.gic.goods.api.dto.ErrorCode;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 统一参数校验
 *
 * @author xuwenqian
 */
@RestControllerAdvice
public class SpringGlobalExceptionHandler {

    private static final Logger log = LoggerFactory.getLogger(SpringGlobalExceptionHandler.class);


    @ExceptionHandler(value = Exception.class)
    public RestResponse handlerException(Exception e) {
        log.error("系统异常：{}", e);
        return RestResponse.failure(ErrorCode.UNKNOW_ERROR.getCode(), ErrorCode.UNKNOW_ERROR.getErrorMessage());
    }

    @ExceptionHandler(value = NullPointerException.class)
    public RestResponse nullExceptionHandler(NullPointerException e) {
        log.error("空指针异常！原因是:{}", e);
        return RestResponse.failure(ErrorCode.NULL_ERROR.getCode(), e.getMessage());
    }

    @ExceptionHandler(value = BaseRuntimeException.class)
    public RestResponse baseRuntimeException(BaseRuntimeException e) {
        log.error("运行异常！原因是:{}", e);
        return RestResponse.failure(e.getErrCode(), e.getMessage());
    }

    @ExceptionHandler(value = CommonException.class)
    public RestResponse enterpriseCommonException(CommonException e) {
        return RestResponse.failure(e.getErrorCode(), e.getMessage());
    }

    /**
     * 统一参数校验
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(BindException.class)
    public RestResponse handleBindException(BindException ex) {
        FieldError fieldError = ex.getBindingResult().getFieldError();
        return RestResponse.failure(ErrorCode.INVALIDATE_PARAM.getCode(), fieldError.getDefaultMessage());
    }

    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public RestResponse handleNotValid(MethodArgumentNotValidException exception) {
        String message = exception.getBindingResult().getFieldError().getDefaultMessage();
        return RestResponse.failure(ErrorCode.INVALIDATE_PARAM.getCode(), message);
    }

    @ExceptionHandler(value = ConstraintViolationException.class)
    public RestResponse methodConstraintViolationException(ConstraintViolationException exception) {
        List<String> msg = new ArrayList<>();
        Set<ConstraintViolation<?>> constraintViolations = exception.getConstraintViolations();
        RestResponse result = new RestResponse();
        result.setCode(ErrorCode.INVALIDATE_PARAM.getCode());
        for (ConstraintViolation<?> constraintViolation : constraintViolations) {
            msg.add(constraintViolation.getMessage());
        }
        result.setMessage(StringUtils.join(msg, ","));
        return result;
    }

}
